//
//  ViewController.swift
//  DeviceControlSample2
//
//  Created by driver on 2018/08/07.
//  Copyright © 2018年 CITIZEN SYSTEMS. All rights reserved.
//

import UIKit
import CSJPOSLibSwift

class ViewController: UIViewController, UITextFieldDelegate, ScannerEventDelegate {
    
    @IBOutlet weak var editText_Address_Wifi: UITextField!
    @IBOutlet weak var segmentPortType: UISegmentedControl!
    @IBOutlet weak var editText_PrinterPort: UITextField!
    @IBOutlet weak var editText_DisplayAddress: UITextField!
    @IBOutlet weak var editText_DisplayPort: UITextField!
    @IBOutlet weak var editText_ScannerAddress: UITextField!
    @IBOutlet weak var editText_ScannerPort: UITextField!
    @IBOutlet weak var lavel_ScannerStatus: UITextField!
    
    var printer: ESCPOSPrinter?
    var display: CSJPOSLibSwift.LineDisplay?
    var scanner: CSJPOSLibSwift.Scanner?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        editText_Address_Wifi.returnKeyType = UIReturnKeyType.done
        editText_Address_Wifi.delegate = self
        
        segmentPortType.selectedSegmentIndex = 0
        
        editText_Address_Wifi.text = ""
        
        editText_PrinterPort.returnKeyType = UIReturnKeyType.done
        editText_PrinterPort.delegate = self
        
        editText_DisplayAddress.returnKeyType = UIReturnKeyType.done
        editText_DisplayAddress.delegate = self
        editText_DisplayAddress.text = ""
        
        editText_DisplayPort.returnKeyType = UIReturnKeyType.done
        editText_DisplayPort.delegate = self
        
        editText_ScannerAddress.returnKeyType = UIReturnKeyType.done
        editText_ScannerAddress.delegate = self
        editText_ScannerAddress.text = ""
        
        editText_ScannerPort.returnKeyType = UIReturnKeyType.done
        editText_ScannerPort.delegate = self
        
        lavel_ScannerStatus.text = "Stopped"
        lavel_ScannerStatus.backgroundColor = UIColor.lightGray
        
        printer = CSJPOSLibSwift.ESCPOSPrinter()
        display = CSJPOSLibSwift.LineDisplay()
        scanner = CSJPOSLibSwift.Scanner()
        
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func messageBox(_ message: String, withTitle title: String, withAutoDismiss dismiss: Bool){
        let alert: UIAlertController = UIAlertController(title:title, message: message, preferredStyle:  UIAlertControllerStyle.alert)
        if dismiss == true {
            present(alert, animated: true, completion: {
                DispatchQueue.main.asyncAfter(deadline: .now() + 1.5, execute: {
                    alert.dismiss(animated: false, completion: nil)
                })
            })
        }
        else {
            let defaultAction: UIAlertAction = UIAlertAction(title: "OK", style: UIAlertActionStyle.default, handler:{
                (action: UIAlertAction!) -> Void in
            })
            alert.addAction(defaultAction)
            present(alert, animated: true, completion: nil)
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        editText_Address_Wifi.resignFirstResponder()
        editText_PrinterPort.resignFirstResponder()
        editText_DisplayAddress.resignFirstResponder()
        editText_DisplayPort.resignFirstResponder()
        editText_ScannerAddress.resignFirstResponder()
        editText_ScannerPort.resignFirstResponder()
        return true
    }
    
    @IBAction func onClick_button_Search(_ sender: Any) {
        var iftype:Int32 = 0
        var result: Int32 = 0
        var addressArray: Any = []
        var address_Array: [String] = []

        if segmentPortType.selectedSegmentIndex == 0 {
            iftype = ESCPOSConst.CMP_PORT_BLUETOOTH
        } else if segmentPortType.selectedSegmentIndex == 1 {
            iftype = ESCPOSConst.CMP_PORT_WiFi
        } else if segmentPortType.selectedSegmentIndex == 2 {
            iftype = ESCPOSConst.CMP_PORT_USB
        }

        addressArray = ESCPOSPrinter.searchESCPOSPrinter(iftype, withSearchTime: Int32(3), result: &result) as Any
        
        if (addressArray as AnyObject).count > 0 {
            for i in 0 ..< (addressArray as AnyObject).count {
                let obj = (addressArray as! NSArray)[i]
                address_Array.append(obj as! String)
            }
        }else{
            messageBox("Search Error : \(result)", withTitle: "0 Search Results", withAutoDismiss: true)
        }
        if address_Array.count > 0 {
            let alert: UIAlertController = UIAlertController(title:"Select the address", message: nil, preferredStyle:  UIAlertController.Style.alert)
            for i in 0 ..< address_Array.count {
                let defaultAction: UIAlertAction = UIAlertAction(title: address_Array[i], style: UIAlertAction.Style.default, handler:{
                    (action: UIAlertAction!) -> Void in
                    self.editText_Address_Wifi.text = address_Array[i]
                    self.editText_DisplayAddress.text = address_Array[i]
                    self.editText_ScannerAddress.text = address_Array[i]
                })
                alert.addAction(defaultAction)
            }
            present(alert, animated: true, completion: nil)
        }
    }
    
    @IBAction func onClick_button_connect(_ sender: Any) {
        let addr = editText_Address_Wifi.text
        let port: Int32? = Int32(editText_PrinterPort.text!)
        var result: Int32 = 0
        var iftype:Int32 = 0

        if segmentPortType.selectedSegmentIndex == 0 {
            iftype = ESCPOSConst.CMP_PORT_BLUETOOTH
        } else if segmentPortType.selectedSegmentIndex == 1 {
            iftype = ESCPOSConst.CMP_PORT_WiFi
        } else if segmentPortType.selectedSegmentIndex == 2 {
            iftype = ESCPOSConst.CMP_PORT_USB
        }

        if port != nil {
            result = printer!.connect(iftype, withAddrress: addr, withPort: Int32(port!))
        }else{
            result = printer!.connect(iftype, withAddrress: addr)
        }
        
        if ESCPOSConst.CMP_SUCCESS == result {
            messageBox("connect() : Success", withTitle: "Success", withAutoDismiss: true)
        } else {
            let exError = printer!.getErrorCodeExtended()
            messageBox("Connect Error : \(result)(\(exError))", withTitle: "Error", withAutoDismiss: true)
        }
    }
    
    @IBAction func onClick_button_disconnect(_ sender: Any) {
        _ = disconnect(printer)
    }
    
    @IBAction func onClick_button_print(_ sender: Any) {
        _ = print(printer)
    }
    
    @IBAction func onClick_button_printerCheck(_ sender: Any) {
        _ = printerCheck(printer)
    }
    
    func disconnect(_ printer: ESCPOSPrinter?) -> Bool {
        var result: Int32 = 0
        var bRet = false
        
        if printer == nil {
            
            messageBox("ESCPOSPrinter() : Instance is not created", withTitle: "Error", withAutoDismiss: true)
            return bRet
        }
        
        result = printer!.disconnect()
        if ESCPOSConst.CMP_SUCCESS == result {
            bRet = true
            messageBox("disconnect() : Success", withTitle: "Success", withAutoDismiss: true)
        } else {
            messageBox("disconnect() error : \(result)", withTitle: "Error", withAutoDismiss: true)
        }
        
        return bRet
    }
    
    func print(_ printer: ESCPOSPrinter?) -> Bool {
        var result: Int32 = 0
        var bRet = false
        
        if printer == nil {
            
            messageBox("ESCPOSPrinter() : Instance is not created", withTitle: "Error", withAutoDismiss: true)
            return bRet
        }
        
        // printer check
        result = printer!.printerCheck()
        if ESCPOSConst.CMP_SUCCESS != result {
            messageBox("printerCheck() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return bRet
        }
        
        // get status
        let status = printer!.status()
        if ESCPOSConst.CMP_STS_NORMAL != status {
            messageBox("status() error : \(status)", withTitle: "Error", withAutoDismiss: true)
            return bRet
        }
        
        // character set
        result = printer!.setEncoding( String.Encoding(rawValue: UInt(String.Encoding.isoLatin1.rawValue)) )
        //result = posPtr!.setEncoding(NSISOLatin1StringEncoding)  // japanese shift-jis
        if ESCPOSConst.CMP_SUCCESS != result {
            messageBox("setEncoding() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return bRet
        }
        
        // start transaction (batch)
        result = printer!.transactionPrint(ESCPOSConst.CMP_TP_TRANSACTION)
        if CMP_SUCCESS != result {
            messageBox("transactionPrint() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return bRet
        }
        
        // print app name
        if let appName = Bundle.main.localizedInfoDictionary?["CFBundleDisplayName"] as? String {
            _ = printer!.printText("\(appName)\n\n", withAlignment:ESCPOSConst.CMP_ALIGNMENT_CENTER, withAttribute:ESCPOSConst.CMP_FNT_DEFAULT, withTextSize:ESCPOSConst.CMP_TXT_1WIDTH|ESCPOSConst.CMP_TXT_1HEIGHT)
        }
        
        // printText
        _ = printer!.printText("- Sample Print -\n", withAlignment:ESCPOSConst.CMP_ALIGNMENT_CENTER, withAttribute:ESCPOSConst.CMP_FNT_DEFAULT, withTextSize:ESCPOSConst.CMP_TXT_1WIDTH|ESCPOSConst.CMP_TXT_2HEIGHT)
        _ = printer!.printText("1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", withAlignment:ESCPOSConst.CMP_ALIGNMENT_RIGHT, withAttribute:ESCPOSConst.CMP_FNT_DEFAULT, withTextSize:CMP_TXT_1WIDTH|CMP_TXT_1HEIGHT)
        
        // start page mode
        _ = printer!.pageModePrint(ESCPOSConst.CMP_PM_PAGE_MODE)
        
        // set offset value in pagemode
        _ = printer!.setPageModeVerticalPosition(0)
        _ = printer!.setPageModeHorizontalPosition(0)
        
        // set print direction in pagemode
        _ = printer!.setPageModePrintDirection(ESCPOSConst.CMP_PD_TOP_TO_BOTTOM)
        
        // set printable area in pagemode
        _ = printer!.setPageModePrintArea("500,0,76,800" )
        
        let receipt = String(format: "%c|4C- Receipt -\n", 27)
        _ = printer!.printNormal(receipt)
        _ = printer!.setPageModePrintArea("260,0,120,800")
        _ = printer!.printText("$ 299.99- \n", withAlignment:ESCPOSConst.CMP_ALIGNMENT_CENTER, withAttribute:ESCPOSConst.CMP_FNT_UNDERLINE|ESCPOSConst.CMP_FNT_BOLD, withTextSize:ESCPOSConst.CMP_TXT_4WIDTH|ESCPOSConst.CMP_TXT_4HEIGHT)
        _ = printer!.setPageModePrintArea("88,0,88,560")
        _ = printer!.printText(" CITIZEN SYSTEMS\n", withAlignment:ESCPOSConst.CMP_ALIGNMENT_RIGHT, withAttribute:ESCPOSConst.CMP_FNT_DEFAULT, withTextSize:ESCPOSConst.CMP_TXT_2WIDTH|ESCPOSConst.CMP_TXT_3HEIGHT)
        //posPtr!.setPageModePrintArea: @"0,0,88,480")
        _ = printer!.setPageModePrintArea("0,0,98,480")
        _ = printer!.printBarCode("123456789012", withSymbology:ESCPOSConst.CMP_BCS_UPCA, withHeight: 64, withWidth:4, withAlignment:ESCPOSConst.CMP_ALIGNMENT_LEFT, withTextPosition:ESCPOSConst.CMP_HRI_TEXT_BELOW)
        _ = printer!.setPageModePrintArea("0,600,192,192")
        _ = printer!.printQRCode("http://www.citizen-systems.co.jp/", withModuleSize :5, withECLevel:ESCPOSConst.CMP_QRCODE_EC_LEVEL_L, withAlignment:ESCPOSConst.CMP_ALIGNMENT_LEFT)
        
        // end page mode
        _ = printer!.pageModePrint(ESCPOSConst.CMP_PM_NORMAL)
        
        // end transaction (batch)
        result = printer!.transactionPrint(ESCPOSConst.CMP_TP_NORMAL)
        
        if ESCPOSConst.CMP_SUCCESS != result {
            messageBox("printText() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            _ = printer!.clearOutput()
            return bRet
        }
        
        // partical cut with pre-feed
        result = printer!.cutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED)
        if ESCPOSConst.CMP_SUCCESS != result {
            messageBox("cutPaper() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            _ = printer!.clearOutput()
            return bRet
        }
        
        // end transaction (batch)
        result = printer!.transactionPrint(ESCPOSConst.CMP_TP_NORMAL)
        if CMP_SUCCESS != result {
            messageBox("transactionPrint() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            _ = printer!.clearOutput()
            return bRet
        }
        
        bRet = true
        
        return bRet
    }
    
    func printerCheck(_ posPtr: ESCPOSPrinter?) -> Bool {
        var result: Int32 = 0
        var bRet = false
        var msg: String? = ""
        
        if printer == nil {
            msg = "ESCPOSPrinter() : Instance is not created"
        } else {
            // printer check
            result = printer!.printerCheck()
            if CMP_SUCCESS == result {
                msg = "\nprinterCheck() : Success\n"
                
                // get status
                let status = printer!.status()
                if ESCPOSConst.CMP_STS_NORMAL == status {
                    
                    bRet = true
                    msg = msg! + "\n\t * Normal ( No Error )"
                } else {
                    
                    if (ESCPOSConst.CMP_STS_COVER_OPEN & status) > 0 {
                        msg = msg! + "\n\t * Cover Open"
                    }
                    if (ESCPOSConst.CMP_STS_PAPER_EMPTY & status) > 0 {
                        msg = msg! + "\n\t * Paper Empty"
                    }
                    if (ESCPOSConst.CMP_STS_PRINTEROFF & status) > 0 {
                        msg = msg! + "\n\t * Printer Offline"
                    }
                }
            } else {
                msg! = "\nprintCheck() error : \(result)"
            }
        }
        
        if msg != nil {
            messageBox(msg!, withTitle: "Status", withAutoDismiss: true)
        }
        
        return bRet
    }
    
    @IBAction func onClick_button_DisplayConnect(_ sender: Any) {
        let addr = editText_DisplayAddress.text
        let port: Int32? = Int32(editText_DisplayPort.text!)
        var result: Int32 = 0
        var iftype:Int32 = 0

        if segmentPortType.selectedSegmentIndex == 0 {
            iftype = LineDisplayConst.CDP_PORT_BLUETOOTH
        } else if segmentPortType.selectedSegmentIndex == 1 {
            iftype = LineDisplayConst.CDP_PORT_WiFi
        } else if segmentPortType.selectedSegmentIndex == 2 {
            iftype = LineDisplayConst.CDP_PORT_USB
        }

        if port != nil {
            result = display!.connect(iftype, withAddrress: addr, withPort: Int32(port!))
        }else{
            result = display!.connect(iftype, withAddrress: addr)
        }
        
        if LineDisplayConst.CDP_SUCCESS == result {
            messageBox("connect() : Success", withTitle: "Success", withAutoDismiss: true)
        } else {
            let exError = display!.getErrorCodeExtended()
            messageBox("Connect Error : \(result)(\(exError))", withTitle: "Error", withAutoDismiss: true)
        }
    }
    
    @IBAction func onClick_button_DisplayDisconnect(_ sender: Any) {
        
        var result: Int32 = 0
        
        if display == nil {
            messageBox("LineDisplay() : Instance is not created", withTitle: "Error", withAutoDismiss: true)
            return
        }
        result = display!.disconnect()
        
        if LineDisplayConst.CDP_SUCCESS == result {
            messageBox("disconnect() : Success", withTitle: "Success", withAutoDismiss: true)
        } else {
            messageBox("disconnect() error : \(result)", withTitle: "Error", withAutoDismiss: true)
        }
    }
    
    @IBAction func onClick_button_SampleDisplay(_ sender: Any) {
        var result: Int32 = 0
        
        if display == nil {
            messageBox("LineDisplay() : Instance is not created", withTitle: "Error", withAutoDismiss: true)
            return
        }
        
        result = display!.clearDisplay(CDP_AREA_ALL)
        if LineDisplayConst.CDP_SUCCESS != result {
            messageBox("clearDisplay() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return
        }
        
        result = display!.displayText("Thank you so much!")
        if LineDisplayConst.CDP_SUCCESS != result {
            messageBox("displayText() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return
        }
        
        result = display!.setCursorPosition(13, y: 2)
        if LineDisplayConst.CDP_SUCCESS != result {
            messageBox("setCursorPosition() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return
        }
        
        result = display!.displayText("$299.99-", reverseFlag: true)
        if LineDisplayConst.CDP_SUCCESS != result {
            messageBox("displayText() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return
        }
    }
    
    @IBAction func onClick_button_ClearDisplay(_ sender: Any) {
        var result: Int32 = 0
        
        if display == nil {
            messageBox("LineDisplay() : Instance is not created", withTitle: "Error", withAutoDismiss: true)
            return
        }
        
        result = display!.clearDisplay(CDP_AREA_ALL);
        if LineDisplayConst.CDP_SUCCESS != result {
            messageBox("clearDisplay() error : \(result)", withTitle: "Error", withAutoDismiss: true)
            return
        }
    }
    
    @IBAction func onClick_button_scannerConnect(_ sender: Any) {
        let addr = editText_ScannerAddress.text
        let port: Int32? = Int32(editText_ScannerPort.text!)
        var result: Int32 = 0
        var iftype:Int32 = 0

        if segmentPortType.selectedSegmentIndex == 0 {
            iftype = ScannerConst.CSC_PORT_BLUETOOTH
        } else if segmentPortType.selectedSegmentIndex == 1 {
            iftype = ScannerConst.CSC_PORT_WiFi
        } else if segmentPortType.selectedSegmentIndex == 2 {
            iftype = ScannerConst.CSC_PORT_USB
        }

        if port != nil {
            result = scanner!.connect(iftype, withAddrress: addr, withPort: Int32(port!))
        }else{
            result = scanner!.connect(iftype, withAddrress: addr)
        }
        
        if ScannerConst.CSC_SUCCESS == result {
            messageBox("connect() : Success", withTitle: "Success", withAutoDismiss: true)
            _ = scanner!.setStatusUpdateEventCallback(self)
            _ = scanner!.setDataEventCallback(self)
        } else {
            let exError = scanner!.getErrorCodeExtended()
            messageBox("Connect Error : \(result)(\(exError))", withTitle: "Error", withAutoDismiss: true)
        }
    }
    
    @IBAction func onClick_button_scannerDisconnect(_ sender: Any) {
        var result: Int32 = 0
        
        if scanner == nil {
            messageBox("Scanner() : Instance is not created", withTitle: "Error", withAutoDismiss: true)
            return
        }
        
        _ = scanner!.setStatusUpdateEventCallback(nil)
        _ = scanner!.setDataEventCallback(nil)
        lavel_ScannerStatus.text = "Stopped"
        lavel_ScannerStatus.backgroundColor = UIColor.lightGray
        
        result = scanner!.disconnect()
        
        if ScannerConst.CSC_SUCCESS == result {
            messageBox("disconnect() : Success", withTitle: "Success", withAutoDismiss: true)
        } else {
            messageBox("disconnect() error : \(result)", withTitle: "Error", withAutoDismiss: true)
        }
    }
    
    func statusUpdateEventCallback(_ status: Int32) {
        if ScannerConst.CSC_SUE_POWER_ONLINE == status {
            lavel_ScannerStatus.text = "Online"
            lavel_ScannerStatus.backgroundColor = UIColor.init(red: 0.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: 1.0)
        } else {
            lavel_ScannerStatus.text = "Off-line"
            lavel_ScannerStatus.backgroundColor = UIColor.red
        }
    }
    
    func dataEventCallback(_ data: Data!) {
        messageBox(String(data: data, encoding: .utf8)!, withTitle: "Scan Data", withAutoDismiss: false)
    }
}

